@@ -190,7 +190,12 @@ class hgclient(object):
             raise ValueError('server already open')
 
         self.server = util.popen(self._args, self._env)
-        self._readhello()
+        try:
+            self._readhello()
+        except error.ServerError:
+            ret, serr = self._close()
+            raise error.ServerError('server exited with status %d: %s'
+                                    % (ret, serr.strip()))
         return self
 
     def close(self):
@@ -200,11 +205,13 @@ class hgclient(object):
         Attempting to call any function afterwards that needs to
         communicate with the server will raise a ValueError.
         """
-        self.server.stdin.close()
-        self.server.wait()
+        return self._close()[0]
+
+    def _close(self):
+        _sout, serr = self.server.communicate()
         ret = self.server.returncode
         self.server = None
-        return ret
+        return ret, serr
 
     def add(self, files=[], dryrun=False, subrepos=False, include=None,
             exclude=None):
@@ -561,8 +568,12 @@ class hgclient(object):
                           close_branch=closebranch, d=date, u=user, l=logfile,
                           I=include, X=exclude, amend=amend)
         out = self.rawcommand(args)
-        rev, node = out.splitlines()[-1].rsplit(b(':'))
-        return int(rev.split()[-1]), node
+        m = re.search(b(r'^committed changeset (\d+):([0-9a-f]+)'), out,
+                      re.MULTILINE)
+        if not m:
+            raise ValueError('revision and node not found in hg output: %r' % out)
+        rev, node = m.groups()
+        return int(rev), node
 
     def config(self, names=[], untrusted=False, showsource=False):
         """Return a list of (section, key, value) config settings from all
